# load required packages
library(testthat)
library(learnr)
library(shiny)
library(shinytest)


# set knitr options
knitr::opts_chunk$set(echo = TRUE, class.source = "bg-success", error = TRUE, comment = '##')

# display.brewer.all(colorblindFriendly = TRUE)
palette <- "YlOrRd"

smp <- find.package(package = "shinymgr")






